@@ -1,3 +1,4 @@
+# frozen_string_literal: false
 require 'test/unit'
 
 class TestKeywordArguments < Test::Unit::TestCase
@@ -370,12 +371,8 @@ class TestKeywordArguments < Test::Unit::TestCase
       break eval("proc {|a:| a}", nil, 'xyzzy', __LINE__)
     end
     assert_raise_with_message(ArgumentError, /missing keyword/, feature7701) {b.call}
-    assert_raise_with_message(ArgumentError, /unknown keyword/, feature7701) {b.call(a:0, b:1)}
-    begin
-      b.call(a: 0, b: 1)
-    rescue => e
-      assert_equal('xyzzy', e.backtrace_locations[0].path)
-    end
+    e = assert_raise_with_message(ArgumentError, /unknown keyword/, feature7701) {b.call(a:0, b:1)}
+    assert_equal('xyzzy', e.backtrace_locations[0].path)
 
     assert_equal(42, b.call(a: 42), feature7701)
     assert_equal([[:keyreq, :a]], b.parameters, feature7701)
@@ -389,6 +386,15 @@ class TestKeywordArguments < Test::Unit::TestCase
     assert_equal([[:keyreq, :a], [:keyrest, :bl]], b.parameters, feature7701)
     assert_raise_with_message(ArgumentError, /missing keyword/, bug8139) {b.call(c: bug8139)}
     assert_raise_with_message(ArgumentError, /missing keyword/, bug8139) {b.call}
+
+    b = assert_nothing_raised(SyntaxError, feature7701) do
+      break eval("proc {|m, a:| [m, a]}", nil, 'xyzzy', __LINE__)
+    end
+    assert_raise_with_message(ArgumentError, /missing keyword/) {b.call}
+    assert_equal([:ok, 42], b.call(:ok, a: 42))
+    e = assert_raise_with_message(ArgumentError, /unknown keyword/) {b.call(42, a:0, b:1)}
+    assert_equal('xyzzy', e.backtrace_locations[0].path)
+    assert_equal([[:opt, :m], [:keyreq, :a]], b.parameters)
   end
 
   def test_super_with_keyword
@@ -604,18 +610,6 @@ class TestKeywordArguments < Test::Unit::TestCase
     }
   end
 
-  def test_non_keyword_hash_subclass
-    bug12884 = '[ruby-core:77813] [Bug #12884]'
-    klass = EnvUtil.labeled_class("Child", Hash)
-    obj = Object.new
-    def obj.t(params = klass.new, d: nil); params; end
-    x = klass.new
-    x["foo"] = "bar"
-    result = obj.t(x)
-    assert_equal(x, result)
-    assert_kind_of(klass, result, bug12884)
-  end
-
   def test_kwrest_overwritten
     bug13015 = '[ruby-core:78536] [Bug #13015]'
 
@@ -635,4 +629,16 @@ class TestKeywordArguments < Test::Unit::TestCase
       klass.new(d: 4)
     end
   end
+
+  def test_non_keyword_hash_subclass
+    bug12884 = '[ruby-core:77813] [Bug #12884]'
+    klass = EnvUtil.labeled_class("Child", Hash)
+    obj = Object.new
+    def obj.t(params = klass.new, d: nil); params; end
+    x = klass.new
+    x["foo"] = "bar"
+    result = obj.t(x)
+    assert_equal(x, result)
+    assert_kind_of(klass, result, bug12884)
+  end
 end
